# ../scripts/custom/gg_teamplay_deathmatch/gg_teamplay_deathmatch.py

# =============================================================================
# >> IMPORTS
# =============================================================================
# Python Imports
#   Path
from path import path

# EventScripts Imports
#   ES
from es import getindexfromhandle
from es import getplayercount
from es import getplayerhandle
from es import getplayerteam
from es import getUseridList
from es import getuserid
from es import isbot
from es import ServerCommand
from es import ServerVar
#   Gamethread
from gamethread import delayed

# GunGame Imports
#   Addons
from gungame51.core.addons.shortcuts import AddonInfo
#   Eventlib
#       Base
from gungame51.core.events.eventlib import ESEvent
#       Resource
from gungame51.core.events.eventlib.resource import ResourceFile
#       Fields
from gungame51.core.events.eventlib.fields import BooleanField
from gungame51.core.events.eventlib.fields import ByteField
from gungame51.core.events.eventlib.fields import ShortField
#   Messaging
from gungame51.core.messaging.shortcuts import langstring
#   Players
from gungame51.core.players import Player
#   Weapons
from gungame51.core.weapons.shortcuts import get_level_multikill
from gungame51.core.weapons.shortcuts import get_level_weapon
from gungame51.core.weapons.shortcuts import get_total_levels

# =============================================================================
# >> ADDON REGISTRATION/INFORMATION
# =============================================================================
info = AddonInfo()
info.name = 'gg_teamplay_deathmatch'
info.title = 'GG Teamplay Deathmatch'
info.author = 'Satoon101'
info.version = '1.0'
info.requires = ['gg_deathmatch']
info.conflicts = ['gg_elimination', 'gg_ffa',
    'gg_handicap', 'gg_teamwork', 'gg_teamplay']
info.translations = ['gg_teamplay_deathmatch']

# =============================================================================
# >> PUBLIC VARIABLE
# =============================================================================
ServerVar(info.name + '_version', info.version,
        info.title + ' Version').set(info.version)
ServerVar(info.name + '_version').makepublic()

# =============================================================================
# >> GLOBAL VARIABLES
# =============================================================================
# Messaging cvars
level_messages = ServerVar('gg_tp_dm_level_messages')
level_info = ServerVar('gg_tp_dm_level_info')
winner_messages = ServerVar('gg_tp_dm_winner_messages')

# Multikill cvars
multikill_type = ServerVar('gg_tp_dm_multikill')
multikill_grenade = ServerVar('gg_tp_dm_multikill_grenade')
multikill_knife = ServerVar('gg_tp_dm_multikill_knife')

# Special Kill cvars
knife_kills = ServerVar('gg_tp_dm_knife')
prop_kills = ServerVar('gg_tp_dm_prop_physics')


# =============================================================================
# >> EVENT CLASSES
# =============================================================================
class GG_Team_Win(ESEvent):
    '''Fires when a team wins the game'''

    winner = ShortField(
        min_value=2, max_value=3, comment='Team that won the match')

    loser = ShortField(
        min_value=2, max_value=3, comment='Team that lost the match')


class GG_Team_LevelUp(ESEvent):
    '''Fires when a team levels up'''

    team = ShortField(min_value=2, max_value=3, comment='Team that leveled up')

    old_level = ByteField(
        min_value=1, comment='The old level of the team that leveled up')

    new_level = ByteField(
        min_value=1, comment='The new level of the team that leveled up')

# Get the ResourceFile instance to create the .res file
gg_teamplay_deathmatch_resource = ResourceFile(
    path(__file__).parent.joinpath('gg_teamplay_deathmatch.res'))

# Write the .res file
gg_teamplay_deathmatch_resource.write(
    [GG_Team_Win, GG_Team_LevelUp], overwrite=True)


# =============================================================================
# >> TEAM CLASSES
# =============================================================================
class GGTeams(dict):
    '''Class to store the 2 teams'''

    def __new__(cls):
        '''Creates the new object and adds the teams to the dictionary'''

        # Make sure there is only one instance of the class
        if not '_the_instance' in cls.__dict__:

            # Get the new object
            cls._the_instance = dict.__new__(cls)

            # Loop through both teams
            for team in (2, 3):

                # Add the team to the dictionary
                cls._the_instance[team] = TeamManagement(team)

        # Return the dictionary
        return cls._the_instance

    def clear(self):
        '''Resets the team level and multikill values'''

        # Loop through both teams
        for team in self:

            # Reset the level and multikill values
            self[team].reset_levels()

    def set_level_multikills(self):
        '''Sets the multikill level for each team'''

        # Loop through both teams
        for team in self:

            # Set the team's multikill value
            self[team].set_level_multikill()


class TeamManagement(object):
    '''Class used to store team values and perform actions on team members'''

    def __init__(self, team):
        '''Fired when the team's instance is initialized'''

        # Store the team
        self.team = team

        # Set the team's level and multikill value
        self.reset_levels()

    def reset_levels(self):
        '''Resets the team's level and multikill values'''

        # Set the values back to start
        self.multikill = 0
        self.level_multikill = 1
        self.level = 1

    def set_level_multikill(self):
        '''Sets the multikill value for the team's level'''

        # Set the multikill to the team's level's multikill
        self.level_multikill = get_level_multikill(self.level)

        # Does the multikill need multiplied
        # by the number of players on the team?
        if int(multikill_type):

            # Get the current level's weapon
            weapon = get_level_weapon(self.level)

            # Is the level's weapon a knife?
            if weapon == 'knife':

                # Does the knife level multikill need multiplied?
                if not int(multikill_knife):

                    # If not, return
                    return

            # Is the level's weapon a grenade?
            if weapon == 'hegrenade':

                # Does the grenade level multikill need multiplied?
                if not int(multikill_grenade):

                    # If not, return
                    return

            # Multiply the multikill by the number of players on the team
            self.level_multikill *= getplayercount(self.team)

    def increase_multikill(self, userid, weapon):
        '''Increases the team's multikill value
            and checks to see if a levelup is needed'''

        # Was the weapon used the team's current weapon?
        if weapon != get_level_weapon(self.level):

            # Was the kill with a knife?
            if weapon == 'knife':

                # Do knife kills count no matter what?
                if not int(knife_kills):

                    # If not, return
                    return

            # Was the kill with a prop?
            elif weapon == 'prop_physics':

                # Do prop kills count no matter what?
                if not int(prop_kills):

                    # If not, return
                    return

            # Was the weapon neither a knife or prop?
            else:

                # If so, return
                return

        # Increase the team's multikill value
        self.multikill += 1

        # Play multikill sound to all players on the team
        self.play_team_sound('multikill')

        # Does the team need leveled up?
        if self.multikill >= self.level_multikill:

            # Increase the team's level
            self.increase_level()

        # Does a message need sent?
        elif int(level_info):

            # Send the multikill increase message
            self.send_increase_multikill_message()

    def increase_level(self):
        '''Increases the team's level and levels up all team memebers'''

        # Increase the team's level
        self.level += 1

        # Reset the team's multikill value
        self.multikill = 0

        # Did the team just win?
        if self.level > get_total_levels():

            GG_Team_Win(winner=self.team, loser=5 - self.team).fire()

            # Do not send messages or increase player levels
            return

        # Fire GG_Team_LevelUp
        GG_Team_LevelUp(team=self.team,
            old_level=self.level - 1, new_level=self.level).fire()

        # Set the team's new level's multikill
        self.set_level_multikill()

        # Play levelup sound to all players on the team
        self.play_team_sound('levelup')

        # Loop through all team members
        for userid in self.team_players:

            # Set the player's level to the team level
            self.set_player_level(userid)

        # Does a message need sent?
        if int(level_messages):

            # Send the message
            self.send_increase_level_message()

    def set_player_level(self, userid):
        '''Sets the player's level to the team's level'''

        # Get the Player instance
        ggPlayer = Player(userid)

        # Is the player prevented from leveling?
        if info.name in ggPlayer.preventlevel.levelup:

            # Remove the prevention
            ggPlayer.preventlevel.levelup.remove(info.name)

        # Set the player's level to the team's level
        ggPlayer.level = self.level

        # Add the addon to the player's levelup preventlevel
        ggPlayer.preventlevel.levelup.append(info.name)

    def send_increase_multikill_message(self):
        '''Sends information to chat when a team's multikill increases'''

        # Get the other team's instance
        other = GGTeams()[5 - self.team]

        # Is the current team leading?
        if self.level > other.level:

            # Set message name
            message = 'TeamPlay_MultiKill_Leading'

            # Get the level difference
            levels = self.level - other.level

        # Is the current team losing?
        elif self.level < other.level:

            # Set message name
            message = 'TeamPlay_MultiKill_Trailing'

            # Get the level difference
            levels = other.level - self.level

        # Are the teams tied?
        else:

            # Set the message name
            message = 'TeamPlay_MultiKill_Tied'

            # Set the level difference
            levels = 0

        # Store a team member's index
        index = self.index

        # Loop through all players on the server
        for userid in getUseridList():

            # Is the player a bot?
            if isbot(userid):

                # If so, do not send the message
                continue

            # Get the leveling team's name
            teamname = langstring(self.teamname, userid=userid)

            # Send the message to the player
            Player(userid).saytext2(index, message,
                {'teamname': teamname, 'multikill': self.multikill,
                 'total': self.level_multikill, 'levels': levels,
                 'level': self.level, 'other': other.level})

    def send_increase_level_message(self):
        '''Sends information to chat when a team increases their level'''

        # Get the other team's instance
        other = GGTeams()[5 - self.team]

        # Did the team just take the lead?
        if self.level - 1 == other.level:

            # Set the message name
            message = 'TeamPlay_Level_TakeLead'

            # Set the level difference
            levels = 1

        # Did the team increase it's lead?
        elif self.level > other.level:

            # Set the message name
            message = 'TeamPlay_Level_Increase'

            # Get the level difference
            levels = self.level - other.level

        # Did the team just tie the game?
        elif self.level == other.level:

            # Set the message name
            message = 'TeamPlay_Level_Tied'

            # Set the level difference
            levels = 0

        # Is the team still losing?
        else:

            # Set the message name
            message = 'TeamPlay_Level_Trailing'

            # Get the level difference
            levels = other.level - self.level

        # Store a team member's index
        index = self.index

        # Loop through all players on the server
        for userid in getUseridList():

            # Is the player a bot?
            if isbot(userid):

                # If so, do not send the message
                continue

            # Get the leveling team's name
            teamname = langstring(self.teamname, userid=userid)

            # Send the message to the player
            Player(userid).saytext2(index, message,
                {'teamname': teamname, 'levels': levels, 'level': self.level})

    def send_hudhint_info(self, userid):
        '''Sends level info on player_spawn'''

        # Is the player a Bot?
        if isbot(userid):

            # Do not send the level info
            return

        # Get the total levels
        total_levels = get_total_levels()

        # Get the team's name
        teamname = langstring(self.teamname, userid=userid)

        # Create text showing team's level
        text = langstring(
            'TeamPlay_LevelInfo_TeamLevel', {'teamname': teamname,
            'level': self.level, 'total': total_levels}, userid)

        # Add the teams weapon to the text
        text += langstring(
            'TeamPlay_LevelInfo_TeamWeapon', {'teamname': teamname,
            'weapon': get_level_weapon(self.level)}, userid)

        # Is more than 1 kill required for this level?
        if self.level_multikill > 1:

            # Add Required Rounds to the text
            text += langstring(
                'TeamPlay_LevelInfo_RequiredKills',
                {'kills': self.multikill,
                'total': self.level_multikill}, userid)

        # Get the other team's instance
        other = GGTeams()[5 - self.team]

        # Add the other team's level information
        text += langstring('TeamPlay_LevelInfo_TeamLevel',
            {'teamname': langstring(other.teamname, userid=userid),
            'level': other.level, 'total': total_levels}, userid)

        # Is more than 1 round required for this level?
        if other.level_multikill > 1:

            # Add Required Rounds to the text
            text += langstring(
                'TeamPlay_LevelInfo_RequiredKills',
                {'kills': other.multikill,
                'total': other.level_multikill}, userid)

        # Remove the last \n
        text = text.rstrip('\\n')

        # Send the player the hudhint
        Player(userid).hudhint(text)

    def send_winner_messages(self):
        '''Sends Winner Messages to all players'''

        # Store a team player's index
        index = self.index

        # Store the team's color
        color = self.color

        # Loop through all players on the server
        for userid in getUseridList():

            # Is the current player a bot?
            if isbot(userid):

                # Do not send messages to bots
                continue

            # Get the player's Player() instance
            ggPlayer = Player(userid)

            # Get the team's name
            teamname = langstring(self.teamname, userid=userid)

            # Send chat message for team winning the match
            ggPlayer.saytext2(index,
                'TeamPlay_Winner', {'teamname': teamname})

            # We want to loop, so we send a message every second for 3 seconds
            for x in xrange(4):

                # Send centermsg about the winner
                delayed(x, ggPlayer.centermsg,
                    ('TeamPlay_Winner_Center', {'teamname': teamname}))

            # Send toptext message about the winner
            ggPlayer.toptext(10, color,
                'TeamPlay_Winner_Center', {'teamname': teamname})

    def play_team_sound(self, sound):
        '''Plays a sound to all members of the team'''

        # Loop through all team members
        for userid in self.team_players:

            # Play the sound to the player
            Player(userid).playsound(sound)

    @property
    def team_players(self):
        '''Returns all userid's on the team'''

        # Loop through all players on the server
        for userid in getUseridList():

            # Is the player on this team?
            if getplayerteam(userid) == self.team:

                # Yield the player's userid
                yield userid

    @property
    def index(self):
        '''Returns the index of a player on the team'''

        # Loop through all players on the server
        for userid in self.team_players:

            # Return the index of the first team player found
            return getindexfromhandle(getplayerhandle(userid))

    @property
    def teamname(self):
        '''Returns the team's name for use with langstring'''

        # Return the team's langstring name
        return 'TeamPlay_DeathMatch_%s' % self.team

    @property
    def color(self):
        '''Returns the team's color'''

        # Is this the Terrorist team?
        if self.team == 2:

            # Return red for Terrorist color
            return '#red'

        # Return blue for Counter-Terrorist color
        return '#blue'


# =============================================================================
# >> LOAD & UNLOAD
# =============================================================================
def load():
    '''Fired when the script is loaded'''

    # Declare and load the resource file
    gg_teamplay_deathmatch_resource.declare_and_load()


def unload():
    '''Fired when the script is unloaded'''

    # Loop through all players on the server
    for userid in getUseridList():

        # Get the Player() instance
        ggPlayer = Player(userid)

        # Does the player have gg_teamplay in preventlevel?
        if info.name in ggPlayer.preventlevel.levelup:

            # Remove the addon from the player's levelup preventlevel
            ggPlayer.preventlevel.levelup.remove(info.name)


# =============================================================================
# >> GAME EVENTS
# =============================================================================
def es_map_start(event_var):
    '''Fired when a new map starts'''

    # Reset team level and multikill values
    GGTeams().clear()

    # Load the resource file
    gg_teamplay_deathmatch_resource.load()


def round_freeze_end(event_var):
    '''Called at the start of every round'''

    # Set the multikill value for each team
    GGTeams().set_level_multikills()


def player_spawn(event_var):
    '''Fired every time a player spawns'''

    # Get the player's team
    team = int(event_var['es_userteam'])

    # Is the player on a team?
    if not team in GGTeams():

        # If not, return
        return

    # Store the userid
    userid = int(event_var['userid'])

    # Set the player's level to the team's level
    GGTeams()[team].set_player_level(userid)

    # Does the player need the level info sent?
    if int(level_info):

        # Send the player their team's level info
        GGTeams()[team].send_hudhint_info(userid)


def player_death(event_var):
    '''Fired when a player dies'''

    # Get the attacker's team
    attackerteam = event_var['es_attackerteam']

    # Was this a teamkill or suicide?
    if attackerteam in (event_var['es_userteam'], '0', ''):

        # If so, return
        return

    # Typecast the attacker's team
    attackerteam = int(attackerteam)

    # Is the attacker on a team?
    if attackerteam in GGTeams():

        # Increase the killer's team's multikill
        GGTeams()[attackerteam].increase_multikill(
            int(event_var['attacker']), event_var['weapon'])


def gg_start(event_var):
    '''Fired when the match is about to start'''

    # Reset team level and multikill values
    GGTeams().clear()


def gg_team_win(event_var):
    '''Fired when a team wins the match'''

    # Reset team level and multikill values
    GGTeams().clear()

    # Send Winner Messages?
    if int(winner_messages):

        # Send Winner Messages
        GGTeams()[int(event_var['winner'])].send_winner_messages()

    # Get a random player from the server
    userid = getuserid()

    # End the match
    ServerCommand('es_xgive %s game_end' % userid)
    ServerCommand('es_xfire %s game_end EndGame' % userid)

    # Loop through all players on the server
    for userid in getUseridList():

        # Is the player a bot?
        if isbot(userid):

            # If so, don't play the sound
            continue

        # Play the winner sound to the player
        Player(userid).playsound('winner')
