# ../scripts/custom/gg_teamplay_deathmatch/gg_teamplay_deathmatch_config.py

# =============================================================================
# >> IMPORTS
# =============================================================================
# Python Imports
from __future__ import with_statement
from path import path

# GunGame Imports
from gungame51.core.cfg.configs import ConfigContextManager


# =============================================================================
# >> LOAD & UNLOAD
# =============================================================================
def load():

    # Create the cfg file
    with ConfigContextManager(
      path(__file__).parent.split('scripts')[~0][1:]) as config:

        # Create the gg_teamplay_deathmatch instance
        with config.cfg_cvar('gg_teamplay_deathmatch') as cvar:

            cvar.name = 'TEAMPLAY DEATHMATCH'
            cvar.description.append(
                'Level-up as a team with constant respawning.')
            cvar.options.append(
                '0 = (Disabled) Do not load gg_teamplay_deathmatch.')
            cvar.options.append('1 = (Enabled) Load gg_teamplay_deathmatch.')
            cvar.default = 0
            cvar.text = 'Enables/Disables gg_teamplay_deathmatch.'

        # Create the multikill section
        config.cfg_section('TEAMPLAY DEATHMATCH MULTIKILLS')

        # Create the gg_tp_dm_multikill instance
        with config.cfg_cvar('gg_tp_dm_multikill') as cvar:

            cvar.name = 'MULTIKILL TYPE'
            cvar.description.append('If enabled, this will multiply the ' +
                'number of kills required to level (the "multikill") ' +
                'by the number of players on the team.')
            cvar.description.append('This means that if your team is on ' +
                'a level with a multikill value of 1, and you have 4 ' +
                'people on your team, your team must get 4 kills ' +
                'with the current weapon to level up.')
            cvar.options.append('0 = Use the multikill value to ' +
                'determine the number of kills required.')
            cvar.options.append('1 = Multiply the multikill by the ' +
                'number of players on the team to determine ' +
                'the number of kills required.')
            cvar.default = 0
            cvar.text = ('Multiply the multikill each level by the ' +
                'number of players on a team to determine ' +
                'how many kills are required to level up.')

        # Create the gg_tp_dm_multikill_grenade instance
        with config.cfg_cvar('gg_tp_dm_multikill_grenade') as cvar:

            cvar.name = 'MULTIKILL GRENADE'
            cvar.description.append(
                'If enabled, hegrenade level will also have the ' +
                'multikill multiplied by the number of players on a team.')
            cvar.options.append('0 = Do not multiply hegrenade multikill ' +
                'amount by the number of players on a team.')
            cvar.options.append('1 = Multiply hegrenade multikill ' +
                'amount by the number of players on a team.')
            cvar.default = 0
            cvar.text = ('Multiply the multikill value of hegrenade ' +
                'level by the number of players on a team.')

        # Create the gg_tp_dm_multikill_knife instance
        with config.cfg_cvar('gg_tp_dm_multikill_knife') as cvar:

            cvar.name = 'MULTIKILL KNIFE'
            cvar.description.append(
                'If enabled, knife level will also have the ' +
                'multikill multiplied by the number of players on a team.')
            cvar.options.append('0 = Do not multiply knife multikill ' +
                'amount by the number of players on a team.')
            cvar.options.append('1 = Multiply knife multikill ' +
                'amount by the number of players on a team.')
            cvar.default = 0
            cvar.text = ('Multiply the multikill value of knife ' +
                'level by the number of players on a team.')

        # Create the special kill section
        config.cfg_section('TEAMPLAY DEATHMATCH SPECIAL KILLS')

        # Create the gg_tp_dm_knife instance
        with config.cfg_cvar('gg_tp_dm_knife') as cvar:

            cvar.name = 'KNIFE KILLS'
            cvar.description.append('If enabled, knife kills will ' +
                "count towards the team's total "
                'needed kills on their current level')
            cvar.options.append('0 = Knife kills do not count.')
            cvar.options.append('1 = Knife kills count.')
            cvar.default = 0
            cvar.text = ('Knife kills count toward a ' +
                "team's total for their current level")

        # Create the gg_tp_dm_prop_physics instance
        with config.cfg_cvar('gg_tp_dm_prop_physics') as cvar:

            cvar.name = 'PROP PHYSICS KILLS'
            cvar.description.append('If enabled, prop_physics kills will ' +
                "count towards the team's total "
                'needed kills on their current level')
            cvar.options.append('0 = prop_physics kills do not count.')
            cvar.options.append('1 = prop_physics kills count.')
            cvar.default = 0
            cvar.text = ('prop_physics kills count toward a ' +
                "team's total for their current level")

        # Create the Messages section
        config.cfg_section('TEAMPLAY DEATHMATCH MESSAGES')

        # Create the gg_tp_dm_level_messages instance
        with config.cfg_cvar('gg_tp_dm_level_messages') as cvar:

            cvar.name = 'LEVEL MESSAGES'
            cvar.description.append(
                'Sends chat messages when a team levels up.')
            cvar.options.append(
                '0 = Do not send chat messages when a team levels up.')
            cvar.options.append(
                '1 = Send chat messages when a team levels up.')
            cvar.default = 0
            cvar.text = 'Sends chat messages when a team levels up.'

        # Create the gg_tp_dm_level_info instance
        with config.cfg_cvar('gg_tp_dm_level_info') as cvar:

            cvar.name = 'LEVEL INFO'
            cvar.description.append('Sends level info messages ' +
                'when a player spawns and multikill info ' +
                'messages when a team increases their multikill.')
            cvar.options.append('0 = Do not send level info messages.')
            cvar.options.append('1 = Send level info messages.')
            cvar.default = 0
            cvar.text = 'Sends level info messages to players on spawn.'

        # Create the gg_tp_dm_winner_messages instance
        with config.cfg_cvar('gg_tp_dm_winner_messages') as cvar:

            cvar.name = 'WINNER MESSAGES'
            cvar.description.append('Sends chat, center, and ' +
                'toptext messages when a team wins the match.')
            cvar.options.append('0 = Do not send winner messages.')
            cvar.options.append('1 = Send winner messages.')
            cvar.default = 0
            cvar.text = 'Sends messages when a team wins the match.'
